/**************************************************************************************
*
*	FileWrapper
*	 2000 James Dessart
*	A class wrapping up all the (useful) file manipulation functions in the
*		MacOS/Carbon File Manager API
*
*	TODO:
*		- set/getposition()		Set/GetFPos()
*		- create/remove()		FSpCreate/Delete()
*		- operator<<(int)		write(int*,4)
*		- operator>>(int)		read(int*,4)
*
*	History:
*		8/1/2000 - Created
*
**************************************************************************************/

#include <string.h>
#include "FileWrapper.h"

FileWrapper::FileWrapper()
{
	myState = kUninitialized;
}

FileWrapper::FileWrapper(FSSpecPtr pPointer,char pMode,bool pDoCreate,OSType pType,OSType pCreator)
{
	FSSpec	aSpec;
	OSErr	theErr;
	
	theErr = ::FSMakeFSSpec(pPointer->vRefNum,pPointer->parID,pPointer->name,&aSpec);
	if(theErr == noErr)	// File exists
	{
		myFSSpec = pPointer;
		myMode = pMode;
		myState = kClosed;
	}
	else if(theErr == fnfErr) // no such file, but FSSpec is good
	{
		if(pDoCreate)
		{
			theErr = ::FSpCreate(&aSpec,pCreator,pType,-1);
			if(theErr==noErr)
			{
				myFSSpec = pPointer;
				myMode = pMode;
				myState = kClosed;
			}
			else
			{
				myState = kCannotExist;
			}
		}
		else
			myFSSpec = pPointer;
			myMode = pMode;
			myState = kDoesNotExist;
	}
	else
	{
		myState = kBadSpec;
	}
}

FileWrapper::~FileWrapper()
{
	close();
}

bool FileWrapper::isInBadState()
{
	if(myState >= kDoesNotExist && myState <= kCannotOpen)
		return true;
	return false;
}

bool FileWrapper::exists()
{
	if(myState<kDoesNotExist || myState>kBadSpec)
		return true;
	return false;
}

bool FileWrapper::isvalid()
{
	if(myState<kUninitialized || myState>kBadSpec)
		return true;
	return false;
}

bool FileWrapper::isopen()
{
	if(myState >= kOpenReady && myState <= kOpenNoWrite)
		return true;
	return false;
}

bool FileWrapper::canwrite()
{
	if(isopen() && myState != kOpenNoWrite && myState != kOpenDiskFull)
		return true;
	else
		return false;
}

void FileWrapper::resetdiskfull()
{
	if(isopen() && myState == kOpenDiskFull)
		myState = kOpenReady;
}

bool FileWrapper::open(char pMode)
{
	if(isopen())
		return true;
	OSErr theErr;
	if(!isInBadState())
	{
		if(pMode==FileWrapper::kAny)
			pMode = myMode;
		theErr = ::FSpOpenDF(myFSSpec,myMode,&myRefNum);
		if(theErr == noErr)
		{
			if(pMode==FileWrapper::kAny || pMode==FileWrapper::kReadWrite)
				myState = kOpenReady;
			else
				myState = kOpenNoWrite;
			return true;
		}
		else
		{
			myState = kCannotOpen;
			return false;
		}
	}
	else
		return false;
}

void FileWrapper::close()
{
	OSErr theErr;
	if(isopen())
	{
		theErr = ::FSClose(myRefNum);
		if(theErr!=noErr)
			; // damn... how did that happen?  I've been very good about error handling.
		myState = kClosed;
	}
}

char FileWrapper::read()
{
	OSErr theErr;
	char theChar;
	long	byte=1;
	if(isopen())
	{
		theErr = ::FSRead(myRefNum,&byte,&theChar);
		if(theErr==noErr&&byte==1)
			return theChar;
		else if(theErr==eofErr)
		{
			myState = kOpenEOF;
			return '\0';
		}
		else if(theErr==fLckdErr)
		{
			myState = kOpenNoWrite;
			return theChar;
		}
		else
			return '\0';
	}
	else
		return '\0';
}

char *FileWrapper::read(long pNumBytes)
{
	OSErr theErr;
	char *theBuffer = new char[pNumBytes];
	long tempbytes=pNumBytes;
	if(isopen())
	{
		theErr = ::FSRead(myRefNum, &pNumBytes, theBuffer);
		if(theErr==noErr && pNumBytes==tempbytes)
			return theBuffer;
		else if (theErr==eofErr)
		{
			theBuffer[pNumBytes]=0;
			return theBuffer;
		}
		else
			return NULL;
	}
	else
		return NULL;
}

bool FileWrapper::write(char pByte)
{
	OSErr theErr;
	long	byte=1;
	if(isopen() && canwrite())
	{
		theErr = ::FSWrite(myRefNum,&byte,&pByte);
		if(theErr==noErr && byte==1)
			return true;
		else
		{
			switch(theErr)
			{
				case dskFulErr:
					myState = kOpenDiskFull;
					break;
				case wPrErr:
				case fLckdErr:
				case vLckdErr:
				case wrPermErr:
					myState = kOpenNoWrite;
					break;
				default:
					myState = kBadIO;
					break;
			}
			return false;
		}
	}
	else
		return false;
}

bool FileWrapper::write(char *pBytes, long pNumBytes)
{
	OSErr theErr;
	long	tempbytes=pNumBytes;
	if(isopen() && canwrite())
	{
		theErr = ::FSWrite(myRefNum,&pNumBytes,pBytes);
		if(theErr==noErr&&pNumBytes==tempbytes)
			return true;
		else
		{
			switch(theErr)
			{
				case dskFulErr:
					myState = kOpenDiskFull;
					break;
				case wPrErr:
				case fLckdErr:
				case vLckdErr:
				case wrPermErr:
					myState = kOpenNoWrite;
					break;
				default:
					myState = kBadIO;
					break;
			}
			return false;
		}
	}
	else
		return false;
}

/*FileWrapper &FileWrapper::operator>>(char &pChar)
{
	if(myState==kOpenEOF)
		return NULL;
	pChar = read();
	return *this;
}

FileWrapper &FileWrapper::operator<<(char pChar)
{
	bool result = write(pChar);
	if(result)
		return *this;
	else
		return NULL;
}

FileWrapper &FileWrapper::operator>>(char *pString)
{
	int i=0;
	char theChar;
	if(myState==kOpenEOF)
	{
		return NULL;
	}
	while((theChar = read())!='\n' && theChar)
	{
		pString[i++]=theChar;
	}
	pString[i]='\0';
	
	return *this;
}

FileWrapper &FileWrapper::operator<<(char *pString)
{
	int size = strlen(pString);
	bool result = write(pString,size);
	if(result)
		return *this;
	else
		return NULL;
}
*/
