/**************************************************************************************
*
*	FileWrapper
*	 2000 James Dessart
*	A class wrapping up all the (useful) file manipulation functions in the
*		MacOS/Carbon File Manager API
*
**************************************************************************************/

#ifndef JD_FILE_WRAPPER
#define JD_FILE_WRAPPER

#ifdef __APPLE__
#include <Carbon/Carbon.h>
#endif

typedef enum
{
	kOpenReady		=0,			// Open states
	kOpenEOF		=1,
	kOpenDiskFull	=2,
	kOpenNoWrite,				// End open states
	
	kDoesNotExist,			// Bad states

	kUninitialized	=5,		// "spec is not trustable/valid" states
	kCannotExist,
	kBadSpec,

	kBadIO,
	kCannotOpen,			// End bad states
			
	kClosed			=10		// Closed state
} FWState;

class FileWrapper
{
	public:
		const static char kAny = fsCurPerm;
		const static char kReadOnly = fsRdPerm;
		const static char kWriteOnly = fsWrPerm;
		const static char kReadWrite = fsRdWrPerm;
		

		FileWrapper(FSSpecPtr pPointer, char pMode = FileWrapper::kAny, bool pDoCreate = false,
				OSType pType='????',OSType pCreator='????');
		FileWrapper();	// Does nothing, primarily for EOF
		~FileWrapper();

		// Use ::FSpDFOpen
		bool open(char pMode = FileWrapper::kAny);	// Returns true on success
		bool isopen();					// Returns true if open

		// Use ::FSClose()
		void close();						// Closes file

        // as of yet, unimplemented...
		bool create();						// Creates the file
		bool remove();						// deletes the file
		
		bool isvalid();						// Returns true if FSSpec is valid
		bool exists();						// Returns true if FSSpec points to a file

		// Use ::FSRead()
		char read();						// Read one byte
		char *read(long pNumBytes);			// Read pNumBytes bytes
		char *readline();					// Read until \n

		// Use ::FSWrite()
		bool write(char pByte);				// Writes a byte to the file
		bool write(char *pBytes, long pNumBytes); // Or pNumBytes

		bool canwrite();
		void resetdiskfull();

		// Use ::SetFPos()
		bool setposition(long pPosition);	// Sets the position of the file
											// Returns true on success
		long getposition();

		void setmode(char pMode) { myMode = pMode; }

/*		FileWrapper &operator>>(int &pInt);	// These assume SameEndian ordering
		FileWrapper &operator<<(int pInt);	// of ints inside files and mem
											// since this uses binary storage
											// ie no stringtonum, just 4bytes
		
		FileWrapper &operator>>(char &pChar);	// Plainly obvious
		FileWrapper &operator<<(char pChar);
		
		FileWrapper &operator>>(char *pString);	// Reads a string from the file
												// until \n
		FileWrapper &operator<<(char *pString);	// Writes a string to the file
                */

		bool operator!=(FileWrapper *pFile) { return pFile == this; }
		short getrefnum() {return myRefNum;}
	protected:
		bool isInBadState();
	private:
		FSSpecPtr	myFSSpec;
		short		myRefNum;
		char		myMode;
		FWState		myState;
};

#endif // JD_FILE_WRAPPER
