#include "Handlers.h"
#include "SRGameObject.h"
#include "SRPlayerSprite.h"
#include "SRSoundManager.h"
#include "SRGraphicsUtils.h"

#include <stdio.h>
#include <string.h>

//#include <iostream>

// from main.h (no circular references)
extern WindowRef   gGameWindow;

EventTargetRef     gGameWindowEventTarget;
// BUG!!!  Read on...
// this is misnamed... it stores whether a menu is down, OR whether the app is in the background
// or whether the about dialog is up... but will change whenever the state of one of these
// things changes... bug...
bool				gTrackingMenu;

pascal OSErr QuitAppleEventHandler(const AppleEvent *, AppleEvent*, long)
{
	QuitApplicationEventLoop();
	return noErr;
}

pascal OSStatus AppCommandHandler(EventHandlerCallRef, EventRef theEvent, void*)
{
    OSStatus        theErr = noErr;
    HICommand       theCommand;
    
  	GetEventParameter( theEvent, kEventParamDirectObject, typeHICommand, 0, 
  	                   sizeof(HICommand), 0, &theCommand);

    switch(theCommand.commandID)
    {
        case kHICommandQuit:
            QuitApplicationEventLoop();
            break;
        
        case kAboutCommand:
            About();
            break;
        
        case kPauseCommand:
            SRGameObject::TogglePause();
            SetPauseItem();
            break;
            
        case kNewGameCommand:
            SRGameObject::instance->NewGame();
            break;
        
        case kMusicCommand:
            SRSoundManager::TogglePlaying();
            SetMusicItem();
            break;
        
        default:
            theErr = eventNotHandledErr;
            break;
    }
    HiliteMenu(0);
    return theErr;
}

pascal OSStatus WindowEventHandler(EventHandlerCallRef, EventRef theEvent, void*)
{
    OSStatus    theErr = eventNotHandledErr;

    if (GetEventKind(theEvent) == kEventWindowDrawContent && !gTrackingMenu)
    {
        CGrafPtr thePort = GetWindowPort(gGameWindow);
		Rect bounds;
		GetWindowPortBounds(gGameWindow, &bounds);
		
		SRRect theBounds(bounds);
		
		if( SRGraphicsUtils::UsingQuickdraw )
		{
			// Yep... we don't want Quickdraw anymore
			abort();
		}
		else
		{
			CGContextRef myContext;
		 
			SetPortWindowPort(gGameWindow);
			QDBeginCGContext( thePort, &myContext );
		 
				CGContextSaveGState(myContext);
				
				CGContextSetRGBFillColor( myContext, 0, 0, 0, 1 );
				CGContextFillRect( myContext, theBounds );
				
				SRGameObject::instance->Draw(myContext);
				
				CGContextFlush(myContext);

				CGContextRestoreGState(myContext);

			 QDEndCGContext( thePort, &myContext );		
		}

        theErr = noErr;
    }
    
    return theErr;
}

pascal void TimerHandler( EventLoopTimerRef, void* )
{
    if(!SRGameObject::Paused() && !gTrackingMenu)
    {
        // do game-related actions
        SRGameObject::instance->Think();
        SRGameObject::instance->Move();
    }
    
    // for every case, send a draw event to the game window, for adequate refreshes
    EventRef    event;
    CreateEvent( 0, kEventClassWindow, kEventWindowDrawContent, 0, kEventAttributeNone, &event );
    SendEventToEventTarget( event, gGameWindowEventTarget );
}

pascal OSStatus KeyHandler(EventHandlerCallRef, EventRef theEvent, void*)
{
    SRPlayerSprite *player = SRPlayerSprite::instance();
    SRPlayerSprite::Command playerCommand;
    OSStatus theErr = noErr;
    UInt32  keymods;
    char    keycode;
    
    GetEventParameter( theEvent, kEventParamKeyMacCharCodes, typeChar, 0, sizeof(char), 0, &keycode );
    GetEventParameter( theEvent, kEventParamKeyModifiers, typeUInt32, 0, sizeof(UInt32), 0, &keymods );
    
    if(keymods & cmdKey)
    {
        return eventNotHandledErr;
    }
    
    // what key did they press?
    switch(keycode)
    {
        case '4':
        case 'a':
        case 'A':
        case kLeftArrowCharCode:
            playerCommand = SRPlayerSprite::DirectionLeft;
            break;
        case '5':
        case '.':
            playerCommand = SRPlayerSprite::DirectionHalt;
            break;
        case '6':
        case 'd':
        case 'D':
        case kRightArrowCharCode:
            playerCommand = SRPlayerSprite::DirectionRight;
            break;
        case '8':
        case 'w':
        case 'W':
        case kUpArrowCharCode:
            playerCommand = SRPlayerSprite::DirectionUp;
            break;
        case '2':
        case 's':
        case 'S':
        case kDownArrowCharCode:
            playerCommand = SRPlayerSprite::DirectionDown;
            break;
        case '0':
        case ' ':
            playerCommand = SRPlayerSprite::WeaponFire;
            break;
        
        default:
            theErr = eventNotHandledErr;
            break;
    }
    
    if( (theErr == noErr) && (!SRGameObject::instance->Paused()) )
    {
        player->TakeCommand(playerCommand);
    }
    
    return theErr;
}

pascal OSStatus MenuTrackHandler( EventHandlerCallRef /*nextHandler*/,
                                  EventRef /*theEvent*/, void* /*userData*/)
{
    OSStatus theErr = noErr;
    
    gTrackingMenu = !gTrackingMenu;

    return theErr;
}

void InstallEventHandlers()
{
    SetMusicItem();

    // Window handler
	EventTypeSpec	type = { kEventClassWindow, kEventWindowDrawContent };
    EventHandlerUPP winEventHandler = NewEventHandlerUPP(WindowEventHandler);
    InstallWindowEventHandler( gGameWindow, winEventHandler, 1, &type, 0, 0 );
    gGameWindowEventTarget = GetWindowEventTarget( gGameWindow );
    
    // Application event handler
    EventTypeSpec   appList[] = {{ kEventClassCommand, kEventCommandProcess }};
    EventHandlerUPP appEventHandler = NewEventHandlerUPP(AppCommandHandler);
    InstallApplicationEventHandler( appEventHandler, 1, appList, 0, 0 );
    
    // Timer handler - will fire every 30th of a second
    EventLoopRef mainLoop;
    EventLoopTimerUPP timerUPP;
    mainLoop = GetMainEventLoop();
    timerUPP = NewEventLoopTimerUPP(TimerHandler);
    InstallEventLoopTimer( mainLoop, (EventTime)(kEventDurationSecond / 2), 
                          (EventTime)(kEventDurationSecond / 30),
                           timerUPP, 0, 0 );
   
    // key events
    EventTypeSpec   list[] = { { kEventClassKeyboard, kEventRawKeyDown },
                               { kEventClassKeyboard, kEventRawKeyRepeat } };
    EventHandlerUPP keyEventUPP = NewEventHandlerUPP(KeyHandler);
    InstallApplicationEventHandler( keyEventUPP, 2, list, 0, 0 );
    
    // menu tracking events
    EventTypeSpec   types[] = { { kEventClassMenu, kEventMenuBeginTracking },
                                 { kEventClassMenu, kEventMenuEndTracking } };
    EventHandlerUPP menuEventHandler = NewEventHandlerUPP(MenuTrackHandler);
    InstallApplicationEventHandler( menuEventHandler, 2, types, 0, 0 );

    // suspend/resume events
    EventTypeSpec   susresume[] = { { kEventClassApplication, kEventAppActivated },
                                    { kEventClassApplication, kEventAppDeactivated } };
    EventHandlerUPP switchEventHandler = NewEventHandlerUPP(MenuTrackHandler);
    InstallApplicationEventHandler( switchEventHandler, 2, susresume, 0, 0 );

    // set to true, because the first event received is a app-activated event...
    // Note: This isn't an issue in MacOS X, only lower numbered versions
	// UGH: In 10.4, it seems that this is working the opposite of the way it should
    gTrackingMenu = false;
}

void SetPauseItem()
{
    MenuRef menu = GetMenuHandle(130);
    CheckMenuItem( menu, 2, SRGameObject::Paused() );
}

void SetMusicItem()
{
    MenuRef menu = GetMenuHandle(130);
    CheckMenuItem( menu, 4, SRSoundManager::IsPlaying() );
}

void About()
{
    bool oldgTrackingMenu = gTrackingMenu;
    gTrackingMenu = false;
    Alert( kAboutAlert, nil );
    gTrackingMenu = oldgTrackingMenu;
}
