#ifndef __SR_ENEMY_COMMAND_H__
#define __SR_ENEMY_COMMAND_H__

#include "SRObject.h"
#include "SREnemySprite.h"
#include "SRObjectTypes.h"

#define     kCurbWidth      5.0f
#define     kBottomBoundary 48.0f

#define     kRightVelocity  2.0f
#define     kLeftVelocity   -2.0f

class SREnemyCommand : public SRObject
{
public:
    // formation is an array of chars following this structure:
    // ----------------------
    // |  Number of Records | 1 byte
    // |--------------------|
    // |  Record 0          | 4 bytes
    // |--------------------|
    //  ...
    // |--------------------|
    // |  Record n          | 4 bytes
    // ----------------------
    // where each record consists of xoffset (1 byte, signed), yoffset (1 byte, signed),
    // blocked-by (1 byte, refers to a record in this formation), ship-type (1 byte)
    // It is assumed that offset of 0,0 is the center of the group
    SREnemyCommand( const char formation[] );
    ~SREnemyCommand();
    
    void ResetPosition();
    
    virtual void Think();
    virtual bool Hit( SRObject *other );
    virtual void ChildDied( SRObject *ship );
    virtual int Type() { return kEnemyCommandType; }
    virtual void Draw( CGContextRef /*theRef = 0*/ ) {}

protected:
    struct ShipPosition
    {
        char            id;
        int            xOffset;
        int            yOffset;
        char            blockedBy;
        bool            isIndy;
        SREnemySprite  *ship;
    };
    
    // an array of ship positions and their associated ships
    ShipPosition    *fShips;
    char            fNumShips;
    char            fNumLivingShips;
};

#endif // __SR_ENEMY_COMMAND_H__
