#include "SREnemySprite.h"
#include "SRAmmoSprite.h"
#include "SRGameObject.h"
#include "SRSoundManager.h"
#include "SRBoomSprite.h"
#include "SRGraphicsUtils.h"

#include "SRSpiderFireSprite.h"

//#include <iostream>

SREnemySprite::SREnemySprite( CFStringRef iconBasename, SRObject *command )
	: fIconSwitchTimer(kSwitchCountdown), fCurrentImageIndex(0), fCanFire(false)
{
    fParent = command;
    
    // Load icons
	SRGraphicsUtils::CreateCGImagesForFramedResourcesNamed( iconBasename, fImages, kEnemyFrameCount );

	fImage = fImages[0];
}

SREnemySprite::~SREnemySprite()
{
    // dump the icons
    int i;
	for( i = 0; i < kEnemyFrameCount; i++ )
	{
		CGImageRelease(fImages[i]);
	}    
}
    
void SREnemySprite::Think()
{
    // decide what clothes to wear
    if( fIconSwitchTimer > 0 )
	{
		fIconSwitchTimer--;
	}
    else
    {
        fIconSwitchTimer = kSwitchCountdown;
        
		fImage = fImages[fCurrentImageIndex];
		
		fCurrentImageIndex = (fCurrentImageIndex + 1) % kEnemyFrameCount;
    }
    
    // if we can fire (no one's in the way), see if we will
    if(fCanFire)
    {
        long randomNum = ::random();
    
        if(randomNum < 0) randomNum = -randomNum;
    
        float percent = float(randomNum) / float(RAND_MAX);
    
        // if we fall under the threshold, we let a missile loose
        if(percent < kFireProbability)
        {
            SRAmmoSprite *ammo;
            if( percent > 0.001 )
            {
                ammo = new SRAmmoSprite( CFSTR("enemy_ammo"), this );
            }
            else
            {
                ammo = new SRSpiderFireSprite( this );
            }
            ammo->setX( fX );
            ammo->setY( fY + kFireDistance );
            ammo->pushY( kFireForce );
        }
    }
	
	SRCgImageSprite::Think();
}

bool SREnemySprite::Hit( SRObject *other )
{
    // if what hit us is enemy fire, we die...
    if( other->Type() == kPlayerAmmoType || other->Type() == kGenericAmmoType )
    {
        SRSoundManager::PlaySoundEffect(SRSoundManager::SFXboom);
        fParent->ChildDied(this);
        SRGameObject::instance->Scored(kScoreValue);
        new SRBoomSprite(this);
        return true;
    }
    
    return false;
}

void SREnemySprite::ChildDied( SRObject */*ammo*/ )
{
    // when ammo dies, we remove it...
    //fAmmoList->remove(ammo);
}
