#include "SRGameObject.h"
#include "SRPlayerSprite.h"
#include "SREnemy1Sprite.h"
#include "SREnemyCommand.h"
#include "SRStar.h"
#include "WindowDefs.h"
#include "SRSoundManager.h"
#include "SRGraphicsUtils.h"

#include "FileWrapper.h"
#include "HFileUtils.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

//#include <iostream>

SRGameObject::GameState SRGameObject::fState = STATE_WAIT;
SRGameObject::GameState SRGameObject::fLastState = STATE_WAIT;
SRGameObject *SRGameObject::instance = 0;

namespace
{
	FileWrapper *dlog = 0;
	static const int lifeUpThreshold = 5000;

    SRPlayerSprite *player;

	static const int playerImmunityFrameCount = 60;

// debugging formation - see SREnemyCommand.h for details of format
const char formation[] = { 5,
                           0, 20, -1, 1,
                           -20, -20, -1, 1,
                           20, -20, -1, 1,
                           -40, 0, -1, 1,
                           40, 0, -1, 1 };
                           
const char blockFormation[] = { 12,
                                -48, 32, -1, 1,
                                -16, 32, -1, 1,
                                16, 32, -1, 1,
                                48, 32, -1, 1,
                                -48, 0, 0, 1,
                                -16, 0, 1, 1,
                                16, 0, 2, 1,
                                48, 0, 3, 1,
                                -48, -32, 4, 1,
                                -16, -32, 5, 1,
                                16, -32, 6, 1,
                                48, -32, 7, 1 };
}

SRGameObject::SRGameObject() : fEnemyCommand(0), fScore(0), fLivesLeft(0), fList(0),
                               fCurrentLevel(1), fPlayerImmuneFrames(0)
{
    instance = this;
    
    /*FSSpec spec = GetCurrentApplicationFSSpec();
    sprintf((char *)&(spec.name[1]), "debug.txt" );
    spec.name[0] = strlen((char *)&(spec.name[1]));

    if(!dlog)
    {
        dlog = new FileWrapper( &spec, FileWrapper::kWriteOnly, true, 'TEXT', 'R*ch' );
        if(!dlog->open()) SysBeep(1);
    }*/

    // Load splash and game over screens
    fSplashScreen = GetPicture( kSplash );
    fGameOverScreen = GetPicture( kGameOver );
    fCurrentScreen = fSplashScreen;

	fCGSplashScreen = SRGraphicsUtils::CreateCGImageForResourceNamed(CFSTR("Splash"));
	fCGGameOverScreen = SRGraphicsUtils::CreateCGImageForResourceNamed(CFSTR("GameOver"));
	fCGCurrentScreen = fCGSplashScreen;

    CreateStarField();

    fList = new SRObjectList(false, true);

    player = new SRPlayerSprite();
    // Testing only
    //new SREnemy1Sprite(140);
    
    // Wait until the user asks for it...
    //NewGame();
}

SRGameObject::~SRGameObject()
{
    if( dlog != 0 )
        dlog->close();
    delete fList;
    delete fStarField;
}

void SRGameObject::NewGame()
{
//    cout << "Starting new game." << endl;
    // Initialize new enemies to play against
    fState = STATE_PLAY;
    SRSoundManager::SetMusic(SRSoundManager::MUSaggravated);
    SRSoundManager::StartMusic();

    SRObjectList::Iterator *it = fList->NewIterator();
    SRObject *currentObject;
    
    while( (currentObject = it->next()) != 0 )
    {
        if( currentObject->Type() != kPlayerType )
        {
            fList->remove(currentObject);
        }
    }
    
    fList->clean();
    
    fScore = fLastScore = 0;
    fLivesLeft = 3;
    fCurrentLevel = 1;

    fEnemyCommand = new SREnemyCommand(formation);
    
    player->ResetPosition();
	
	fPlayerImmuneFrames = 0;
}

void SRGameObject::LevelUp()
{
    // Initialize next level
    SRObjectList::Iterator *it = fList->NewIterator();
    SRObject *currentObject;
    
    while( (currentObject = it->next()) != 0 )
    {
        if( currentObject->Type() != kPlayerType && 
            currentObject->Type() != kEnemyAmmoType &&
            currentObject->Type() != kExplosionType )
        {
            fList->remove(currentObject);
        }
    }
    
    fList->clean();

    fCurrentLevel++;

    fEnemyCommand = new SREnemyCommand(blockFormation);
	
	fPlayerImmuneFrames = 0;
}

void SRGameObject::GameOver()
{
    // ignore a game over if we aren't playing
    if( fState != STATE_PLAY ) return;
    
    SysBeep(1);

    // the actual screen will be drawn later
    fCGCurrentScreen = fCGGameOverScreen;
    fState = STATE_WAIT;
    
    // for now, a game over will not trigger a new game
    //NewGame();
}

void SRGameObject::PlayerDied()
{
	fLivesLeft--;
	
	player->ResetPosition();
	
	fPlayerImmuneFrames = playerImmunityFrameCount;
}

// The following three methods delegate their functions to the 
// objects in the object list
void SRGameObject::Draw( CGContextRef theRef )
{
    if(fState == STATE_PLAY) ObscureCursor();

    if( fState != STATE_WAIT )
    {
        SRObject *currentObject;
        SRObjectList::Iterator *it = fStarField->NewIterator();
        
		// Draw the star field
        while( (currentObject = it->next()) != 0 )
        {
            currentObject->Draw(theRef);
        }
        delete it;
        
        it = fList->NewIterator();
        
		// Draw the active objects
        while( (currentObject = it->next()) != 0 )
        {
			// If we're busy counting down the player's death immunity,
			// then we need to think about whether or not to draw
			if( currentObject->Type() == kPlayerType && fPlayerImmuneFrames )
			{
				if( (fPlayerImmuneFrames >= 82) ||
					(fPlayerImmuneFrames < 74 && fPlayerImmuneFrames >= 66) ||
					(fPlayerImmuneFrames < 58 && fPlayerImmuneFrames >= 50) ||
					(fPlayerImmuneFrames < 42 && fPlayerImmuneFrames >= 34) ||
					(fPlayerImmuneFrames < 30 && fPlayerImmuneFrames >= 26) ||
					(fPlayerImmuneFrames < 22 && fPlayerImmuneFrames >= 14) ||
					(fPlayerImmuneFrames < 10 && fPlayerImmuneFrames >= 6) ||
					(fPlayerImmuneFrames <  4 && fPlayerImmuneFrames >= 2) )
				{
					currentObject->Draw(theRef);
				}
			}
			else
			{
				currentObject->Draw(theRef);
			}
        }
        delete it;
    }
    else
    {
        DrawScreen(theRef);
    }
    
    char score[255];
    char paused[255];
	char lives[255];
	
	sprintf( paused, "Paused..." );
	sprintf( score, "Score: %d", fScore );
	sprintf( lives, (fLivesLeft == 1) ? "%d Life":"%d Lives", fLivesLeft );

	if( SRGraphicsUtils::UsingQuickdraw || theRef == 0 )
	{
		// No more Mr. Nice Guy! Quickdraw is dead!
		abort();
	}
	else
	{
		CGContextSelectFont( theRef, "Geneva", 14, kCGEncodingMacRoman );
		CGContextSetTextDrawingMode( theRef, kCGTextFill );
	
		// If it's paused...
		if( fState == STATE_PAUSED )
		{
			float baseHeight = kGameWindowHeight/3.0 + kGameStatusBarHeight;
		
			// Draw it in red
			CGContextSetRGBFillColor( theRef, 1, 0, 0, 1 );
			CGContextShowTextAtPoint( theRef, kGameWindowWidth/2.0, baseHeight,
									  paused, strlen(paused) );

			// Draw it in yellow
			CGContextSetRGBFillColor( theRef, 1, 1, 0, 1 );
			CGContextShowTextAtPoint( theRef, kGameWindowWidth/2.0 + 2, baseHeight -1,
									  paused, strlen(paused) );
		}
		
		CGContextSetRGBFillColor( theRef, 0, 0, 1, 1 );
		
		CGContextShowTextAtPoint( theRef, 8, 8, score, strlen(score) );
		CGContextShowTextAtPoint( theRef, kGameWindowWidth/2.0, 8, lives, strlen(lives) );
	}
}

void SRGameObject::Think()
{
    fLastScore = fScore;
    if( fState == STATE_PLAY )
    {
        SRObject *currentObject;
        SRObjectList::Iterator *it = fStarField->NewIterator();
        //Str255 ewhat;
        
        while( (currentObject = it->next()) != 0 )
        {
            currentObject->Think();
        }
        delete it;
        
        it = fList->NewIterator();
        
        while( (currentObject = it->next()) != 0 )
        {
            currentObject->Think();
        }
        
        it->reset();
        
        //dlog->write( "--- frame ---\n", strlen("--- frame ---\n") );
        
        while( (currentObject = it->next()) != 0 )
        {
            SRObjectList *hitList = 0;
            bool wasHit = false;
            
			// If the player is currently immune (from dying recently),
			// and we're dealing with the player object, hitList will be 0.
			// Mentioned here because the counter is decremented below.
            if( currentObject->WantsCollisions() )
            {
                hitList = TestForHit(currentObject);
                if(hitList == 0) continue;
                
                // traverse hit list, telling the currentObject that it hit those
                SRObjectList::Iterator *hit = hitList->NewIterator();
                SRObject *object;
                
                while( (object = hit->next()) != 0 )
                {
                    if(currentObject->Hit(object))
                    {
                        wasHit = true;
                        break;
                    }
                }
                
                // all hit objects must be deleted
                hit->reset();
                
                while( (object = hit->next()) != 0 )
                {
                    if(object->Type() != kPlayerType)
                    {
                        fList->remove(object);
                        //delete object;
                    }
                }
                
                delete hit;
                delete hitList;
                
                // if at least one of the objects caused this one to die,
                // we have to delete it
                if(wasHit && (currentObject->Type() != kPlayerType))
                {
                    fList->remove(currentObject);
                    //delete currentObject;
                }
            }
        }
        
        delete it;
        fList->clean();
        
		// Make sure to decrement the counter
		if( fPlayerImmuneFrames > 0 ) fPlayerImmuneFrames--;
		
        SRSoundManager::MusicIdle();
    }
    
    // if the player is out of lives...
    if(fLivesLeft < 1)
    {
        GameOver();
    }
    
    // if the score has changed...
    if(fScore > fLastScore)
    {
        if((fScore/lifeUpThreshold) > (fLastScore/lifeUpThreshold))
        {
            fLivesLeft++;
        }
    }
}

void SRGameObject::Move()
{
    if( fState == STATE_PLAY )
    {
        SRObject *currentObject;
        SRObjectList::Iterator *it = fStarField->NewIterator();
        
        while( (currentObject = it->next()) != 0 )
        {
            currentObject->Move();
        }
        delete it;    
        
        it = fList->NewIterator();
        
        while( (currentObject = it->next()) != 0 )
        {
            currentObject->Move();
        }
        delete it;
    }
}

// This is the meat of the collision detection
// The detection is rectangle based, based on the width/height of
// the two objects being considered.
SRObjectList *SRGameObject::TestForHit( SRObject *querant )
{
	// Ignore hits to the player while the player is immune
	if( querant->Type() == kPlayerType && fPlayerImmuneFrames )
	{
		return 0;
	}
	
    //dlog->write( ">\n", 2 );
    //char string[255];
    bool hit = false;

    SRObject *currentObject;
    SRObjectList *hitList;
    SRObjectList::Iterator *it;

    hitList = new SRObjectList(false, false);
    
    it = fList->NewIterator();
    
    //char type[5];
    //sprintf( string, "---\nquerant: 0x%x\n", querant );
    //dlog->write(string, strlen(string));
    
    //type[4] = 0;
    //*((int*)type) = querant->Type();
    //sprintf( string, "type:   %s\n", type );
    //dlog->write( string, strlen(string) );

    while( (currentObject = it->next()) != 0 )
    {
        // ignore enemy command objects, they have no physical presence
        if( currentObject->Type() == kEnemyCommandType ||
            currentObject->Type() == kExplosionType ||
            currentObject->Type() == kStarfieldType )
        {
            continue;
        }
    
		// Ignore hits to the player while the player is immune
		if( currentObject->Type() == kPlayerType && fPlayerImmuneFrames )
		{
			continue;
		}
	
        if( currentObject != querant  )
        {
            // what is the distance between the two objects, in x and y
            int yDiff = querant->yPosition() - currentObject->yPosition();
            int xDiff = querant->xPosition() - currentObject->xPosition();
            
            // these are the minimum distances between the centers of the
            // two sprites
            int heightTotal = (querant->height() + currentObject->height())/2;
            int widthTotal = (querant->width() + currentObject->width())/2;
            
            bool xHit = false, yHit = false;

            // check if the two sprites are vertically close enough
            if( ((yDiff >= 0) && (yDiff <= heightTotal)) ||
                ((yDiff < 0) && (-yDiff <= heightTotal)) )
            {
                yHit = true;
            }

            // check if the two sprites are horizontally close enough
            if( ((xDiff >= 0) && (xDiff <= widthTotal)) ||
                ((xDiff < 0) && (-xDiff <= widthTotal)) )
            {
                xHit = true;
            }
            
            // were the two sprites close enough on both axes?
            if( xHit && yHit )
            {
                hit = hit || true;

                //type[4] = 0;
                //*((int*)type) = currentObject->Type();
                //sprintf( string, "hit by: 0x%x\n", currentObject );
                //dlog->write( string, strlen(string) );
                //sprintf( string, "type:   %s\n", type );
                //dlog->write( string, strlen(string) );
                // yes, then we'll tell the second object it was hit by the querant
                // and add it to our list of victims
                if( currentObject->Hit(querant) )
                    hitList->add(currentObject);
            }
        }
    }
    
    delete it;

    //dlog->write( "<\n", 2 );
    if(hit) return hitList;
    else
    {
        delete hitList;
        return 0;
    }
}

// This function recycles a parent-less object
void SRGameObject::Recycle(SRObject *object)
{
    fList->remove(object);
    //delete object;
}

void SRGameObject::TogglePause()
{
    switch(fState)
    {
        case STATE_PAUSED:
            fState = fLastState;
            break;
        case STATE_PLAY:
        case STATE_WAIT:
            fLastState = fState;
            fState = STATE_PAUSED;
            break;
    }
    
    if(fState != STATE_PLAY)
    {
        SRSoundManager::PauseSoundEffects();
        SRSoundManager::PauseMusic();
    }
    else
    {
        SRSoundManager::ResumeSoundEffects();
        SRSoundManager::ResumeMusic();
    }
}

void SRGameObject::DrawScreen( CGContextRef theRef )
{
    Rect bounds;
    SetRect( &bounds, 0, 0, kGameWindowWidth, kGameWindowHeight );

	SRRect theBounds( 0, kGameStatusBarHeight, kGameWindowWidth, kGameWindowHeight );

	if( SRGraphicsUtils::UsingQuickdraw || theRef == 0 )
	{
		abort();
	}
	else
	{
		CGContextDrawImage( theRef, theBounds, fCGCurrentScreen );
		
		if( fCGCurrentScreen == fCGGameOverScreen )
		{
			char buffer[255];
			
			sprintf( buffer, "%d", fScore );
			CGContextSelectFont( theRef, "Geneva", 14, kCGEncodingMacRoman );
			CGContextSetTextDrawingMode( theRef, kCGTextFill );
	
			// draw score in blue
			CGContextSetRGBFillColor( theRef, 0, 0, 1, 1 );
			CGContextShowTextAtPoint( theRef, kFinalScoreX, kFinalScoreY, buffer, strlen(buffer) );

			// draw score in green
			CGContextSetRGBFillColor( theRef, 0, 1, 0, 1 );
			CGContextShowTextAtPoint( theRef, kFinalScoreX - 2, kFinalScoreY + 1,
									  buffer, strlen(buffer) );
		}
	}
}

void SRGameObject::CreateStarField()
{
    fStarField = new SRObjectList( false, true );
    
    int i,j;
    long randomNumber;
    float rands[6];
    SRStar *star;
    RGBColor colour;
    
    for( i=0; i<kNumStars; i++ )
    {
        for( j=0; j<6; j++ )
        {
            randomNumber = ::random();
        
            // invert it
            if(randomNumber < 0) randomNumber = -randomNumber;
        
            // generate a number from 0 to 1.0
            rands[j] = 0;
            if(randomNumber != 0) rands[j] = randomNumber / float(RAND_MAX);
        }
        
        colour.red = (UInt16) ( 32767 - (rands[0] * kColorShift) );
        colour.green = (UInt16) ( 32767 - (rands[1] * kColorShift) );
        colour.blue = (UInt16) ( 32767 - (rands[2] * kColorShift) );
        
        star = new SRStar( kGameWindowWidth * rands[3], kGameWindowHeight * rands[4], colour );
		
		if( SRGraphicsUtils::UsingQuickdraw )
		{
			star->pushY( kMaxStarSpeed * rands[5] );
		}
		else
		{
			// Core Graphics goes the opposite direction
			star->pushY( -(kMaxStarSpeed * rands[5]) );
		}
        
        fStarField->add(star);
    }
}
