#include "SRSoundManager.h"

//#include <Sound.h>

SndChannelPtr   SRSoundManager::fSoundEffectsChannel = 0;
SndChannelPtr   SRSoundManager::fSoundEffectsChannelOne = 0;
SndChannelPtr   SRSoundManager::fSoundEffectsChannelTwo = 0;
SndChannelPtr   SRSoundManager::fMusicChannel = 0;
SndListHandle   *SRSoundManager::fSFXArray = 0;
SndListHandle   *SRSoundManager::fMUSArray = 0;
SRSoundManager::Music  SRSoundManager::fCurrentMusic;
bool            SRSoundManager::fMusicIsPlaying;
bool            SRSoundManager::fMusicOn;
bool            SRSoundManager::fFXOn = true;

namespace
{
    struct SFXtoName
    {
        SRSoundManager::SoundEffect fx;
        ConstStringPtr              name;
    };

    struct MUStoName
    {
        SRSoundManager::Music       music;
        ConstStringPtr              name;
    };
    
    SFXtoName sfxnames[] = { {SRSoundManager::SFXboom,"\pSFXboom"},
                             {SRSoundManager::SFXplayerHit,"\pSFXplayerHit"},
                             {SRSoundManager::SFXplayerFires,"\pSFXplayerFires"},
                             {SRSoundManager::SFXammoBoom,"\pSFXammoBoom"} };

    MUStoName musnames[] = { {SRSoundManager::MUSpeaceful,"\pMUSpeaceful"},
                             {SRSoundManager::MUSaggravated,"\pMUSaggravated"},
                             {SRSoundManager::MUSmournful,"\pMUSmournful"} };
}

void SRSoundManager::InitSound()
{
    // Prepare the channels
    OSErr theErr;
    
    fSoundEffectsChannelOne = new SndChannel;
    fSoundEffectsChannelOne->qLength = 128;
    theErr = SndNewChannel( &fSoundEffectsChannelOne, 0, 0, 0 );

    fSoundEffectsChannelTwo = new SndChannel;
    fSoundEffectsChannelTwo->qLength = 128;
    theErr = SndNewChannel( &fSoundEffectsChannelTwo, 0, 0, 0 );

    fSoundEffectsChannel = fSoundEffectsChannelOne;
    
    fMusicChannel = new SndChannel;
    fMusicChannel->qLength = 128;
    theErr = SndNewChannel( &fMusicChannel, 0, 0, NewSndCallBackUPP(MusicCallBack) );
    
    // Load the sounds & music
    fSFXArray = new SndListHandle[SFXnum];
    int i;
    for( i=0; i<SFXnum; i++ )
    {
        fSFXArray[i] = (struct SndListResource**)GetNamedResource( 'snd ', sfxnames[i].name );
    }

    fMUSArray = new SndListHandle[MUSnum];
    for( i=0; i<MUSnum; i++ )
    {
        fMUSArray[i] = (struct SndListResource**)GetNamedResource( 'snd ', musnames[i].name );
    }
}

void SRSoundManager::EndSound()
{
    OSErr theErr;
    
    theErr = SndDisposeChannel( fSoundEffectsChannelOne, true );
    theErr = SndDisposeChannel( fSoundEffectsChannelTwo, true );
    theErr = SndDisposeChannel( fMusicChannel, true );
}
    
void SRSoundManager::PlaySoundEffect( SoundEffect effect )
{
    if(!fFXOn) return;
    
    SndCommand    theCmd;
    theCmd.cmd = quietCmd;
    SndDoImmediate( fSoundEffectsChannel, &theCmd );

    SndPlay( fSoundEffectsChannel, fSFXArray[effect], true );
    
    if( fSoundEffectsChannel == fSoundEffectsChannelOne )
    {
        fSoundEffectsChannel = fSoundEffectsChannelTwo;
    }
    else
    {
        fSoundEffectsChannel = fSoundEffectsChannelOne;
    }
}

void SRSoundManager::PauseSoundEffects()
{
    SndCommand    theCmd;
    theCmd.cmd = pauseCmd;
    
    SndDoImmediate( fSoundEffectsChannelOne, &theCmd );
    SndDoImmediate( fSoundEffectsChannelTwo, &theCmd );
}

void SRSoundManager::ResumeSoundEffects()
{
    SndCommand    theCmd;
    theCmd.cmd = resumeCmd;

    SndDoImmediate( fSoundEffectsChannelOne, &theCmd );
    SndDoImmediate( fSoundEffectsChannelTwo, &theCmd );
}
    
void SRSoundManager::EndSoundEffects()
{
    SndCommand    theCmd;

    theCmd.cmd = flushCmd;
    SndDoImmediate( fSoundEffectsChannelOne, &theCmd );
    SndDoImmediate( fSoundEffectsChannelTwo, &theCmd );
    
    theCmd.cmd = quietCmd;
    SndDoImmediate( fSoundEffectsChannelOne, &theCmd );
    SndDoImmediate( fSoundEffectsChannelTwo, &theCmd );
}

void SRSoundManager::ToggleFX()
{
    fFXOn = !fFXOn;
    
    if(!fFXOn)
    {
        EndSoundEffects();
    }
}

void SRSoundManager::SetMusic( Music music )
{
    fCurrentMusic = music;
}

void SRSoundManager::MusicIdle()
{
    if(fMusicIsPlaying && fMusicOn)
    {
        fMusicIsPlaying = false;
        SndPlay( fMusicChannel, fMUSArray[fCurrentMusic], true );
        SndCommand    theCmd;
        
        theCmd.cmd = waitCmd;
        theCmd.param1 = kMusicWaitTime;
        SndDoCommand( fMusicChannel, &theCmd, false );

        theCmd.cmd = callBackCmd;
        SndDoCommand( fMusicChannel, &theCmd, false );
    }
}

pascal void SRSoundManager::MusicCallBack(SndChannelPtr chan,SndCommand *)
{
    if(chan == fMusicChannel)
    {
        fMusicIsPlaying = true;
    }
}

void SRSoundManager::TogglePlaying()
{
    fMusicOn = !fMusicOn;

    if(fMusicOn)
    {
        StartMusic();
    }
    else
    {
        EndMusic();
    }
}

void SRSoundManager::PauseMusic()
{
    SndCommand    theCmd;
    theCmd.cmd = pauseCmd;

    SndDoImmediate( fMusicChannel, &theCmd );
}

void SRSoundManager::ResumeMusic()
{
    SndCommand    theCmd;
    theCmd.cmd = resumeCmd;

    SndDoImmediate( fMusicChannel, &theCmd );
}

void SRSoundManager::StartMusic()
{
    fMusicIsPlaying = true;
}

void SRSoundManager::EndMusic()
{
    SndCommand    theCmd;

    theCmd.cmd = flushCmd;
    SndDoImmediate( fMusicChannel, &theCmd );
    
    theCmd.cmd = quietCmd;
    SndDoImmediate( fMusicChannel, &theCmd );
    fMusicIsPlaying = false;
}
